@@ -327,6 +327,25 @@ IOSCSIProtocolInterface::CheckPowerState ( void )
 #endif
 
 
+//
+//  willTerminate - Called prior to phase2 termination.			[PROTECTED]
+//
+
+bool
+IOSCSIProtocolInterface::willTerminate ( IOService *   provider,
+										IOOptionBits   options )
+{
+
+	bool	result = false;
+
+	fCommandGate->commandWakeup ( &fCurrentPowerState, false );
+	result = super::willTerminate ( provider, options );
+
+	return result;
+
+}
+ 
+ 
 //
 //  finalize - Terminates all power management.					[PROTECTED]
 //
