@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -23,8 +21,8 @@
  * @APPLE_LICENSE_HEADER_END@
  */
 
-#ifndef _SCSI_COMMAND_OPERATION_CODES_H
-#define _SCSI_COMMAND_OPERATION_CODES_H
+#ifndef _SCSI_COMMAND_OPERATION_CODES_H_
+#define _SCSI_COMMAND_OPERATION_CODES_H_
 
 #pragma mark About this file
 /* This file contains the operation code definitions for all commands defined
@@ -867,4 +865,4 @@ enum
 };
 #endif
 
-#endif	/* _SCSI_COMMAND_OPERATION_CODES_H */
\ No newline at end of file
+#endif	/* _SCSI_COMMAND_OPERATION_CODES_H_ */
\ No newline at end of file
