@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -38,10 +36,10 @@
 #include <IOKit/storage/IOBlockStorageDriver.h>
 
 // SCSI Architecture Model Family includes
-#include <IOKit/scsi-commands/SCSICommandDefinitions.h>
+#include <IOKit/scsi/SCSICommandDefinitions.h>
+#include <IOKit/scsi/IOReducedBlockServices.h>
 
-#include <IOKit/scsi-commands/IOReducedBlockServices.h>
-#include <IOKit/scsi-commands/IOSCSIReducedBlockCommandsDevice.h>
+#include "IOSCSIReducedBlockCommandsDevice.h"
 #include "SCSIReducedBlockCommands.h"
 
 
@@ -117,28 +115,7 @@ IOSCSIReducedBlockCommandsDevice::SyncReadWrite (
 					UInt64					startBlock,
 					UInt64					blockCount )
 {
-	
-	IODirection		direction;
-	IOReturn		status = kIOReturnBadArgument;
-	
-	direction = buffer->getDirection ( );
-	
-	if ( direction == kIODirectionIn )
-	{
-		
-		status = IssueRead ( buffer, startBlock, blockCount );
-		
-	}
-	
-	else if ( direction == kIODirectionOut )
-	{
-		
-		status = IssueWrite ( buffer, startBlock, blockCount );
-		
-	}
-	
-	return status;
-	
+	return kIOReturnUnsupported;
 }
 
 
@@ -189,8 +166,9 @@ IOReturn
 IOSCSIReducedBlockCommandsDevice::EjectTheMedia ( void )
 {
 	
-	SCSITaskIdentifier		request				= NULL;
 	IOReturn				status				= kIOReturnNoResources;
+	SCSIServiceResponse		serviceResponse		= kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE;
+	SCSITaskIdentifier		request				= NULL;
 	bool					doPollForRemoval	= false;
 	
 	STATUS_LOG ( ( "%s::%s called\n", getName ( ), __FUNCTION__ ) );
@@ -206,10 +184,40 @@ IOSCSIReducedBlockCommandsDevice::EjectTheMedia ( void )
 	if ( fMediaIsRemovable == false )
 	{
 		
-		// Not a removable disk. Synchronize the drive's write cache.
-		status = SynchronizeCache ( );
+		if ( getProperty ( "Power Off" ) != NULL )
+		{
+			
+			// Spin down the media now. We use to queue up a power change but we found
+			// that since the PM stuff happens at a deferred point, the machine might shutdown
+			// before we finish the PM change. So, we do the spindown now, then sync with PM
+			// so it knows the state.
+			if ( fCurrentPowerState > kRBCPowerStateSleep )
+			{
+				
+				request = GetSCSITask ( );
+				require_nonzero ( request, ErrorExit );
+				
+				// At a minimum, make sure the drive is spun down
+				if ( START_STOP_UNIT ( request, 1, 0, 0, 0 ) == true )
+				{
+					
+					serviceResponse = SendCommand ( request, 0 );
+					
+				}
+				
+				// Give the drive some time to park the heads.
+				IOSleep ( 500 );
+				
+				ReleaseSCSITask ( request );
+				request = NULL;
+				
+			}
+			
+			fCurrentPowerState = kRBCPowerStateSleep;
+			
+		}
 		
-		// Tell power management to ask us to spin the drive down.
+		// Sync ourselves with PM.
 		changePowerStateToPriv ( kRBCPowerStateSleep );
 		
 	}
@@ -241,9 +249,7 @@ IOSCSIReducedBlockCommandsDevice::EjectTheMedia ( void )
 			// Eject it.
 			if ( START_STOP_UNIT ( request, 0, 0, 1, 0 ) == true )
 			{
-
-				SCSIServiceResponse		serviceResponse;
-
+				
 				// The command was successfully built, now send it
 				serviceResponse = SendCommand ( request, kTenSecondTimeoutInMS );
 			
@@ -511,13 +517,18 @@ IOSCSIReducedBlockCommandsDevice::ReportMaxReadTransfer (
 						kSCSIProtocolFeature_MaximumReadTransferByteCount,
 						&maxByteCount );	
 	
-	if ( ( supported == true ) && ( maxByteCount > 0 ) && ( fMediaBlockSize > 0 ) )
+	if ( ( supported == true ) && ( maxByteCount > 0 ) )
 	{
 		
-		maxBlockCount = min ( maxBlockCount, ( maxByteCount / fMediaBlockSize ) );
+		setProperty ( kIOMaximumByteCountReadKey, maxByteCount, 64 );
+		
+		if ( fMediaBlockSize > 0 )
+			maxBlockCount = min ( maxBlockCount, ( maxByteCount / fMediaBlockSize ) );
 		
 	}
 	
+	setProperty ( kIOMaximumBlockCountReadKey, maxBlockCount, 64 );
+	
 	*max = maxBlockCount * blockSize;
 	
 	return kIOReturnSuccess;
@@ -555,13 +566,18 @@ IOSCSIReducedBlockCommandsDevice::ReportMaxWriteTransfer (
 						kSCSIProtocolFeature_MaximumWriteTransferByteCount,
 						&maxByteCount );	
 	
-	if ( ( supported == true ) && ( maxByteCount > 0 ) && ( fMediaBlockSize > 0 ) )
+	if ( ( supported == true ) && ( maxByteCount > 0 ) )
 	{
 		
-		maxBlockCount = min ( maxBlockCount, ( maxByteCount / fMediaBlockSize ) );
+		setProperty ( kIOMaximumByteCountWriteKey, maxByteCount, 64 );
+		
+		if ( fMediaBlockSize > 0 )
+			maxBlockCount = min ( maxBlockCount, ( maxByteCount / fMediaBlockSize ) );
 		
 	}
 	
+	setProperty ( kIOMaximumBlockCountWriteKey, maxBlockCount, 64 );
+	
 	*max = maxBlockCount * blockSize;
 	
 	return kIOReturnSuccess;
@@ -719,8 +735,7 @@ IOSCSIReducedBlockCommandsDevice::InitializeDeviceSupport ( void )
 	STATUS_LOG ( ( "%s::%s setupSuccessful = %d\n", getName ( ),
 					__FUNCTION__, setupSuccessful ) );
 	
-	setProperty ( kIOMaximumBlockCountReadKey,  kDefaultMaxBlocksPerIO, 64 );
-	setProperty ( kIOMaximumBlockCountWriteKey, kDefaultMaxBlocksPerIO, 64 );
+	SetMediaCharacteristics ( 0, 0 );
 	
 	return setupSuccessful;
 	
@@ -749,27 +764,9 @@ void
 IOSCSIReducedBlockCommandsDevice::StartDeviceSupport ( void )
 {
 	
-	OSBoolean *		shouldNotPoll = NULL;
-	
-	shouldNotPoll = OSDynamicCast (
-							OSBoolean,
-							getProperty ( kAppleKeySwitchProperty ) );
-	
-	if ( shouldNotPoll != NULL )
-	{
-		
-		// See if we should not poll.
-		require ( shouldNotPoll->isFalse ( ), Exit );
-		
-	}
-	
 	// Start polling
 	EnablePolling ( );
 	
-	
-Exit:
-	
-	
 	CreateStorageServiceNub ( );
 	
 }
@@ -1289,8 +1286,6 @@ IOSCSIReducedBlockCommandsDevice::SetMediaCharacteristics (
 	
 	UInt64		maxBytesRead	= 0;
 	UInt64		maxBytesWrite	= 0;
-	UInt64		maxBlocksRead	= 0;
-	UInt64		maxBlocksWrite	= 0;
 	
 	STATUS_LOG ( ( "mediaBlockSize = %ld, blockCount = %ld\n",
 					blockSize, blockCount ) );
@@ -1300,18 +1295,7 @@ IOSCSIReducedBlockCommandsDevice::SetMediaCharacteristics (
 	
 	ReportMaxReadTransfer  ( fMediaBlockSize, &maxBytesRead );
 	ReportMaxWriteTransfer ( fMediaBlockSize, &maxBytesWrite );
-	
-	if ( fMediaBlockSize > 0 )
-	{
 		
-		maxBlocksRead 	= maxBytesRead / fMediaBlockSize;
-		maxBlocksWrite	= maxBytesWrite / fMediaBlockSize;
-		
-		setProperty ( kIOMaximumBlockCountReadKey, maxBlocksRead, 64 );
-		setProperty ( kIOMaximumBlockCountWriteKey, maxBlocksWrite, 64 );
-		
-	}
-	
 }
 
 
@@ -1387,6 +1371,23 @@ IOSCSIReducedBlockCommandsDevice::PollForMedia ( void )
 			bool					validSense		= false;
 			SCSIServiceResponse		serviceResponse;
 			
+			OSBoolean *				keySwitchLocked = NULL;
+			
+			keySwitchLocked = OSDynamicCast (
+									OSBoolean,
+									getProperty ( kAppleKeySwitchProperty ) );
+			
+			if ( keySwitchLocked != NULL )
+			{
+				
+				// See if we should poll for media.
+				if ( keySwitchLocked->isTrue ( ) )
+				{
+					break;
+				}
+				
+			}
+			
 			serviceResponse = kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE;
 			
 			bufferDesc = IOMemoryDescriptor::withAddress ( ( void * ) &senseBuffer,
@@ -1907,7 +1908,7 @@ IOSCSIReducedBlockCommandsDevice::SetMediaIcon ( void )
 
 
 //
-//	 IssueRead - Issues a synchronous read command.				[PROTECTED]
+//	 IssueRead - DEPRECATED.										[PROTECTED]
 //
 
 IOReturn
@@ -1916,56 +1917,7 @@ IOSCSIReducedBlockCommandsDevice::IssueRead (
 									UInt64					startBlock,
 									UInt64					blockCount )
 {
-	
-	SCSIServiceResponse 	serviceResponse = kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE;
-	SCSITaskIdentifier		request			= NULL;
-	IOReturn				status 			= kIOReturnNoResources;
-	
-	request = GetSCSITask ( );
-	require_nonzero ( request, ErrorExit );
-	
-	if ( READ_10 ( 	request,
-					buffer,
-				 	fMediaBlockSize,
-					( SCSICmdField4Byte ) startBlock,
-					( SCSICmdField2Byte ) blockCount ) == true )
-	{
-		
-		// The command was successfully built, now send it
-		serviceResponse = SendCommand ( request, fReadTimeoutDuration );
-		
-		if ( ( serviceResponse == kSCSIServiceResponse_TASK_COMPLETE ) &&
-			 ( GetTaskStatus ( request ) == kSCSITaskStatus_GOOD ) )
-		{
-			
-			status = kIOReturnSuccess;
-			
-		}
-		
-		else
-		{
-			
-			status = kIOReturnIOError;
-			
-		}
-		
-	}
-	
-	else
-	{
-		
-		status = kIOReturnBadArgument;
-		
-	}
-	
-	ReleaseSCSITask ( request );
-	
-	
-ErrorExit:
-	
-	
-	return status;
-	
+	return kIOReturnUnsupported;
 }
 
 
@@ -2022,7 +1974,7 @@ ErrorExit:
 
 
 //
-//	 IssueRead - Issues a synchronous write command.				[PROTECTED]
+//	 IssueWrite - DEPRECATED.										[PROTECTED]
 //
 
 IOReturn
@@ -2031,62 +1983,12 @@ IOSCSIReducedBlockCommandsDevice::IssueWrite (
 						UInt64					startBlock,
 						UInt64					blockCount )
 {
-	
-	SCSIServiceResponse 	serviceResponse = kSCSIServiceResponse_SERVICE_DELIVERY_OR_TARGET_FAILURE;
-	SCSITaskIdentifier		request			= NULL;
-	IOReturn				status 			= kIOReturnNoResources;
-	
-	request = GetSCSITask ( );
-	require_nonzero ( request, ErrorExit );
-	
-	if ( WRITE_10 (	request,
-					buffer,
-					fMediaBlockSize,
-					0,
-					( SCSICmdField4Byte ) startBlock,
-					( SCSICmdField2Byte ) blockCount ) == true )
-	{
-		
-		// The command was successfully built, now send it
-		serviceResponse = SendCommand ( request, fWriteTimeoutDuration );
-		
-		if ( ( serviceResponse == kSCSIServiceResponse_TASK_COMPLETE ) &&
-			 ( GetTaskStatus ( request ) == kSCSITaskStatus_GOOD ) )
-		{
-			
-			status = kIOReturnSuccess;
-			
-		}
-		
-		else
-		{
-			
-			status = kIOReturnIOError;
-			
-		}
-		
-	}
-	
-	else
-	{
-		
-		status = kIOReturnBadArgument;
-		
-	}
-	
-	ReleaseSCSITask ( request );
-	
-	
-ErrorExit:
-	
-	
-	return status;
-	
+	return kIOReturnUnsupported;
 }
 
 
 //
-//	 IssueRead - Issues an asynchronous write command.				[PROTECTED]
+//	 IssueWrite - Issues an asynchronous write command.			[PROTECTED]
 //
 
 IOReturn
@@ -2138,6 +2040,74 @@ ErrorExit:
 }
 
 
+//
+//	 AsyncReadWriteCompletion - Completion routine for read/write requests.
+//															 		[PROTECTED]
+//
+
+void
+IOSCSIReducedBlockCommandsDevice::AsyncReadWriteCompletion (
+										SCSITaskIdentifier completedTask )
+{
+	
+	IOReturn	status		= kIOReturnSuccess;
+	UInt64		actCount	= 0;
+	void *		clientData	= NULL;
+	
+	// Extract the client data from the SCSITaskIdentifier
+	clientData = GetApplicationLayerReference ( completedTask );
+	require_nonzero ( clientData, ErrorExit );
+	
+	if ( ( GetServiceResponse ( completedTask ) == kSCSIServiceResponse_TASK_COMPLETE ) &&
+		 ( GetTaskStatus ( completedTask ) == kSCSITaskStatus_GOOD ) )
+	{
+		
+		// Our status is good, so return a success
+		actCount = GetRealizedDataTransferCount ( completedTask );
+		
+	}
+	
+	else
+	{
+		
+		// Set a generic IO error for starters
+		status = kIOReturnIOError;
+		
+		// Either the task never completed or we have a status other than GOOD,
+		// return an error.		
+		if ( GetTaskStatus ( completedTask ) == kSCSITaskStatus_CHECK_CONDITION )
+		{
+			
+			SCSI_Sense_Data		senseDataBuffer;
+			bool				senseIsValid;
+			
+			senseIsValid = GetAutoSenseData ( completedTask, &senseDataBuffer, sizeof ( senseDataBuffer ) );
+			if ( senseIsValid )
+			{
+				
+				ERROR_LOG ( ( "READ or WRITE failed, ASC = 0x%02x, ASCQ = 0x%02x\n",
+				senseDataBuffer.ADDITIONAL_SENSE_CODE,
+				senseDataBuffer.ADDITIONAL_SENSE_CODE_QUALIFIER ) );
+				
+			}
+			
+		}
+		
+	}
+	
+	IOReducedBlockServices::AsyncReadWriteComplete ( clientData, status, actCount );
+	
+	ReleaseSCSITask ( completedTask );	
+	
+	
+ErrorExit:
+	
+	
+	return;
+	
+}
+
+
 #if 0
 #pragma mark -
 #pragma mark  Static Methods
@@ -2155,73 +2125,19 @@ IOSCSIReducedBlockCommandsDevice::AsyncReadWriteComplete (
 										SCSITaskIdentifier request )
 {
 	
-	void *								clientData 	= NULL;
-	IOSCSIReducedBlockCommandsDevice *	taskOwner	= NULL;
-	SCSITask *							scsiRequest = NULL;
-	IOReturn							status		= kIOReturnIOError;
-	UInt64								actCount 	= 0;
-		
-	scsiRequest = OSDynamicCast ( SCSITask, request );
-	require_nonzero ( scsiRequest, ErrorExit );
+	IOSCSIReducedBlockCommandsDevice *	taskOwner = NULL;
 	
-	taskOwner = OSDynamicCast ( IOSCSIReducedBlockCommandsDevice,
-								scsiRequest->GetTaskOwner ( ) );
+	require_nonzero ( request, ErrorExit );
+	
+	taskOwner = OSDynamicCast ( IOSCSIReducedBlockCommandsDevice, sGetOwnerForTask ( request ) );
 	require_nonzero ( taskOwner, ErrorExit );
 	
-	// Extract the client data from the SCSITask
-	clientData = scsiRequest->GetApplicationLayerReference ( );
-	require_nonzero ( clientData, ErrorExit );
-	
-	if ( ( scsiRequest->GetServiceResponse ( ) == kSCSIServiceResponse_TASK_COMPLETE ) &&
-		 ( scsiRequest->GetTaskStatus ( ) == kSCSITaskStatus_GOOD ) )
-	{
-		
-		// Our status is good, so return a success
-		status = kIOReturnSuccess;
-		actCount = scsiRequest->GetRealizedDataTransferCount ( );
-		
-	}
-	
-	else
-	{
-		
-		// Set a generic IO error for starters
-		status = kIOReturnIOError;
-		
-		// Either the task never completed or we have a status other than GOOD,
-		// return an error.		
-		if ( scsiRequest->GetTaskStatus ( ) == kSCSITaskStatus_CHECK_CONDITION )
-		{
-			
-			SCSI_Sense_Data		senseDataBuffer;
-			bool				senseIsValid;
-			
-			senseIsValid = scsiRequest->GetAutoSenseData ( &senseDataBuffer, sizeof ( senseDataBuffer ) );
-			if ( senseIsValid )
-			{
-				
-				ERROR_LOG ( ( "READ or WRITE failed, ASC = 0x%02x, ASCQ = 0x%02x\n",
-				senseDataBuffer.ADDITIONAL_SENSE_CODE,
-				senseDataBuffer.ADDITIONAL_SENSE_CODE_QUALIFIER ) );
-				
-			}
-			
-		}
-		
-	}
-	
-	IOReducedBlockServices::AsyncReadWriteComplete ( clientData, status, actCount );
-	
-	taskOwner->ReleaseSCSITask ( request );
-	
-	return;
+	taskOwner->AsyncReadWriteCompletion ( request );
 	
 	
 ErrorExit:
 	
 	
-	IOPanic ( "SAM RBC: error completing I/O due to bad completion data" );
-	
 	return;
 	
 }
@@ -2274,9 +2190,9 @@ ErrorExit:
 
 OSMetaClassDefineReservedUsed ( IOSCSIReducedBlockCommandsDevice, 1 );	/* PowerDownHandler	*/
 OSMetaClassDefineReservedUsed ( IOSCSIReducedBlockCommandsDevice, 2 );	/* SetMediaIcon		*/
+OSMetaClassDefineReservedUsed ( IOSCSIReducedBlockCommandsDevice, 3 );	/* AsyncReadWriteCompletion	*/
 
 // Space reserved for future expansion.
-OSMetaClassDefineReservedUnused ( IOSCSIReducedBlockCommandsDevice,  3 );
 OSMetaClassDefineReservedUnused ( IOSCSIReducedBlockCommandsDevice,  4 );
 OSMetaClassDefineReservedUnused ( IOSCSIReducedBlockCommandsDevice,  5 );
 OSMetaClassDefineReservedUnused ( IOSCSIReducedBlockCommandsDevice,  6 );
